/********************************************************************
*	Fondamenti di Informatica - Definizione della Classe Linea		*
*	Esempi:															*
*		1) Disegno casuale di linee									*
*		2) Disegno di linee fissati due punti						*	
*		3) Simulazione della lancetta dei secondi di un orologio	*
********************************************************************/


#include "graphics.h"
#include "conio.h"
#include "stdlib.h"
#include "stdio.h"
#include "math.h"
#include "dos.h"


#include "point.h"


/*******************************************************************
	AGGREGAZIONE: Un punto e' parte della linea
	 _______			 ______
	|		|is-part-of |	   |
	| Point |---------->| Line |
	|_______|			|______|


*******************************************************************/
class Line
{
private:
	Point  ini; // punto di applicazione
	Point  fin; // punto finale
	int color;	// colore del punto

public:
	Line();
	Line(Line &);
	Line(Point &ini, Point &fin);
	Line(Point &ini, Point &fin, int color);
//Metodi per il disegno
	void draw();	         // disegno della linea
	void erase();	         // // cancellatura ( dallo schermo ) della linea
	void move(Point *);      // movimento della linea relativo al punto in ingresso
//Metodi selettori
	void set(Point &ini, Point &fin, int &color);// Settaggio linea

	Point get_ini();// restituisce ini
	Point get_fin();// restituisce fin
	int get_color();// restituisce il numero di colore

// Operatori della classe Line

	int operator==(Line &a);  // confronto uguaglianza nei punti iniziale e finali della linea
};



/************* Metodi della classe Line *********************/


Line::Line()  : ini(0,0), fin(0,0), color(0) { }

Line::Line(Point &i, Point &f)  {ini=i; fin=f;}

Line::Line(Point &i, Point &f, int col) { ini=i; fin=f; color=col; }

Line::Line(Line &l): ini(l.get_ini()), fin(l.get_fin()), color(l.get_color()) {}

void Line::draw()
{
	setcolor((short)color);
	moveto((short)ini.Get_x(),(short)ini.Get_y());
	lineto((short)fin.Get_x(),(short)fin.Get_y());
}

void Line::erase()
{         setcolor(getbkcolor());
	//setcolor((short)FONDO);
	moveto((short)ini.Get_x(),(short)ini.Get_y());
	lineto((short)fin.Get_x(),(short)fin.Get_y());
}

void Line::move(Point *p) { ini=ini+ (*p); fin=fin+ (*p); }

void Line::set(Point &i, Point &f, int &col) { ini=i; fin=f; color=col; }

Point Line::get_ini() { return (ini); }

Point Line::get_fin() { return (fin); }

int Line::get_color() { return color; }


int Line::operator==(Line &a) { return( a.ini==ini && a.fin==fin ); }




/*******************************************************/

#define marg 20  // definisce un valore per il margine

class Schermo
{
protected:
	int X0, Y0;		// attributi relativi alle coordinate
					// del centro dello schermo

	int xmax, ymax; // attributi relativi alle coordinate max
					// dello schermo
					// (0,0) coincide con l'angolo in alto a dx

	void init();	// Metodo di inizializzazione si puo' accedere
					// solo dalla classe schermo
public:
	Schermo();		//Costruttore di default
	void assi();	//Metodo per il disegno degli assi cartesiani
	int apri_grafica(); //Metodo per l'impostazione del modo grafico
	
	int get_X0();	// Metodo selettore: restituisce X0		
	int get_Y0();	// Metodo selettore: restituisce Y0
	
	void cornice();	// Disegna una cornice
	void assi_cornice();	//Disegna una cornice + gli assi
	void orologio();		//Disegna un orologio
}
//Implementazione Metodi

Schermo::Schermo()
{
	X0=0;
	Y0=0;
	xmax=0;
	ymax=0;
}

void Schermo::init()
{
	X0=getmaxx()/2;
	Y0=getmaxy()/2;
	xmax=getmaxx();
	ymax=getmaxy();
}

/*	Tipo enumerato presente nel file di libreria GRAPHICS.H

enum COLORS {
    BLACK,
    BLUE,
    GREEN,
    CYAN,           // Celeste
    RED,
    MAGENTA,        // Violetto
    BROWN,
    LIGHTGRAY,
    DARKGRAY,
    LIGHTBLUE,
    LIGHTGREEN,
    LIGHTCYAN,
    LIGHTRED,
    LIGHTMAGENTA,
    YELLOW,
    WHITE
};              */


void Schermo::assi()
{
	setcolor(WHITE);
	moveto(X0,3*marg);
	lineto(X0,ymax-3*marg);
	moveto(3*marg,Y0);
	lineto(xmax-3*marg,Y0);
}

int Schermo::apri_grafica()     //metodo che permette il passaggio
{                               //alla modalit grafica.
	int gd=DETECT,gm;
	initgraph(&gd,&gm,"\\TC\\BGI"); //funzione per inizializzare il modo grafico
									//gd=tipo di device
									//gm=modo grafico
	if(graphresult()==grOk)		//funzione che ritorna vero se il modo
								//grafico e' stato attivato correttamente
	{
		setgraphmode(VGAHI);	// definisce il modo grafico da usare
		init();
		return 1;
	}
	else return 0;
}

void Schermo::cornice()
{
	cleardevice();				// Pulisce lo schermo
	setcolor(LIGHTGREEN);		// Definisce il colore da usare per disegnare
	rectangle(marg,marg,xmax-marg,ymax-marg); //funzione per il disegno di
											  //un rettangolo
	setcolor(CYAN);
	rectangle(2*marg,2*marg,xmax-2*marg,ymax-2*marg);
}

int Schermo::get_X0() { return (X0); }

int Schermo::get_Y0() { return (Y0); }

void Schermo::assi_cornice()
{
	cornice();
	assi();
}

void Schermo::orologio()
{
	int k=0,i=0;
	long x,y;
	float raggio=150;
	int stop=0;
	Line l1;

	cleardevice();
	setcolor(WHITE);
	circle(X0,Y0,raggio+10); // funzione per il disegno di un cerchio
	while(stop==0)
	{

		x=long (raggio*(cos(3.14*(k/30.-0.5))));
		y=long (raggio*(sin(3.14*(k/30.-0.5))));
		l1.set(Point(get_X0()+x,get_Y0()+y),
			Point(get_X0(),get_Y0()),YELLOW);
		l1.draw();
		k++;
		i=k%60;
		delay(990);	// funzione che genera un'attesa 990 sono i millisecondi
		gotoxy(70,20);	//funzione per posizionare il cursore sullo schermo
						// in modalita' grafica il cursore e' spento
		printf("%02d:%02d\n",k/60,i);
		l1.erase();
		sound(440); //funzione per l'emissione di un suono dallo speaker
					//l'argomento e' la frequenza del suono (Hz)
		delay(10);
		nosound();	//funzione per terminare l'emissione del suono
	    if(kbhit()) stop=1;
	}

}

/*******************************************************/

void main()
{
	char c=' ', opt=' ';
	Point p1,p2,p3(100,100);
	Line l1;
	Schermo scr;
	int x,y;
	int i=0, j=0, esci=0;

	if(!scr.apri_grafica())
	{
		printf("Errore nell'apertura della modalit grafica");
		exit(0);
	}
	 while(esci==0)
	{
		cleardevice();
		gotoxy(28,10);
		printf("Fondamenti di Informatica\n");
		gotoxy(25,11);
		printf("Ingegneria Ambiente e Territorio");
		gotoxy(10,14);
		printf("1) Esempio dispense");
		gotoxy(10,16);
		printf("2) Disegno di linee");
		gotoxy(10,18);
		printf("3) Orologio");
		gotoxy(10,20);
		printf("4) Uscita");
		opt=getch();
		switch(opt)
		{
		  case '1':
		    cleardevice();
		    scr.assi_cornice();
		    for(i=0;i<10;i++)
		    {
			p1.set(rand()/100, rand()/100);
			p2.set(rand()/100, rand()/100);
			l1.set(p3+p1,p3+p2, (j++)%16);
			l1.draw();
		    }
		    getch();
		    break;
		  case '2':
		    while(c!='n')
		    {
		     gotoxy(1,1);
		     cleardevice();
		     printf("Inserire le coordinate del punto X1:\n");
		     printf("x=");
		     scanf("%d",&x);
		     printf("y=");
		     scanf("%d",&y);
		     p1=Point(scr.get_X0()+x,scr.get_Y0()-y);
		     printf("Inserire le coordinate del punto X2:\n");
		     printf("x=");
		     scanf("%d",&x);
		     printf("y=");
		     scanf("%d",&y);
		     p2=Point(scr.get_X0()+x,scr.get_Y0()-y);
		     l1.set(p1,p2,YELLOW);
		     scr.assi_cornice();
		     l1.draw();
		     getch();
		     l1.erase();
		     scr.assi();
		     gotoxy(30,15);
		     printf("UN'ALTRA LINEA? (s/n)\n");
		     c=getch();
		    }
		    break;
		case '3':
		    scr.orologio();
		    break;
		case '4':
		    esci=1;
		    break;
		default:
		    break;
		}
	}
 }